//-------------------------------------------------------------
// Recursive Bush by Andreas Gustafsson (Shadow/Noice) (C) 2000
//-------------------------------------------------------------
#include "stdafx.h"
#include "Texture.h"

#define RECURSION_LEVELS 7

//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Skeleton";
HWND hwnd;

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;

// Use the demonstration texture.
CTexture gTexExample;
CTexture gTexExample2;
int updateFPS=0;
double curFPS=0;
float angle;

float random(float high)
{
    return((float)((double)rand()/32768)*high);
}

// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	glViewport (0, 0, (GLsizei) g_w, (GLsizei) g_h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective(55.0, (GLfloat) g_w/(GLfloat) g_h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.0, 0.0, -3.0);
	glClearColor(0.0,0.0,0.0,0.0);
	glEnable(GL_DEPTH_TEST);

	gTexExample.LoadJPG( "texture.jpg" );
	gTexExample2.LoadJPG( "sand.jpg" );

	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

void drawRec(int levels)
{
	levels--;
	glColor3f(0.6f*(float)levels/(float)RECURSION_LEVELS,
			  0.3f+0.2f*(float)RECURSION_LEVELS/(float)levels,
			  0.4f*(float)levels/(float)RECURSION_LEVELS);

	glScalef(0.7f,0.7f,0.7f);
	glPushMatrix();
	glRotatef(0.0f,1.0f,0.0f,0.0f);
	glRotatef(levels*3.0f,0.0f,1.0f,0.0f);
	glRotatef(35.0f,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,0.6f,0.0f);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-0.125,-0.5,0);
	glTexCoord2f(1,0);
	glVertex3f(0.125,-0.5,0);
	glTexCoord2f(1,1);
	glVertex3f(0.1f,0.5f,0.0f);
	glTexCoord2f(0,1);
	glVertex3f(-0.1f,0.5f,0.0f);
	glEnd();
	if(levels>0)
	{
		drawRec(levels);
	}
	glPopMatrix();
	
	glColor3f(0.6f*(float)levels/(float)RECURSION_LEVELS,
			  0.3f+0.2f*(float)RECURSION_LEVELS/(float)levels,
			  0.4f*(float)levels/(float)RECURSION_LEVELS);
	glPushMatrix();
	glRotatef(0.0f-levels*3,1.0f,0.0f,0.0f);
	glRotatef(123,0.0f,1.0f,0.0f);
	glRotatef(32.0f,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,0.6f,0.0f);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-0.125,-0.5,0);
	glTexCoord2f(1,0);
	glVertex3f(0.125,-0.5,0);
	glTexCoord2f(1,1);
	glVertex3f(0.1f,0.5f,0.0f);
	glTexCoord2f(0,1);
	glVertex3f(-0.1f,0.5f,0.0f);
	glEnd();
	if(levels>1)
	{
		drawRec(levels);
	}
	glPopMatrix();

	glColor3f(0.6f*(float)levels/(float)RECURSION_LEVELS,
			  0.3f+0.2f*(float)RECURSION_LEVELS/(float)levels,
			  0.4f*(float)levels/(float)RECURSION_LEVELS);
	glPushMatrix();
	glRotatef(0.0f+levels*3,1.0f,0.0f,0.0f);
	glRotatef(-109,0.0f,1.0f,0.0f);
	glRotatef(39.0f,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,0.6f,0.0f);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-0.125,-0.5,0);
	glTexCoord2f(1,0);
	glVertex3f(0.125,-0.5,0);
	glTexCoord2f(1,1);
	glVertex3f(0.1f,0.5f,0.0f);
	glTexCoord2f(0,1);
	glVertex3f(-0.1f,0.5f,0.0f);
	glEnd();
	if(levels>1)
	{
		drawRec(levels);
	}
	glPopMatrix();
}

void Shutdown() // Called right after the window is destroyed.
{
}

// Draw all the scene related stuff.
void Render()
{
	char s[100];

	curFPS+=1.f/fDeltaTime;
	updateFPS++;
	if(updateFPS==20)
	{
		sprintf(s,"Recursive Bush by Andreas Gustafsson (Shadow/Noice) (C) 2000   -=[ FPS:%.0f ]=-",curFPS/20.0f);
		SetWindowText(hwnd,s);
		updateFPS=0;
		curFPS=0;
	}

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	glRotatef((float)(25.0+10.0*sin(fTime*0.5)), 1.0f, 0.0f, 0.0f); 
	angle=angle+30*fDeltaTime;
	if(angle>360.0f) angle-=360.0f;
	glTranslatef(0.0f,0.0f,-0.5f+1.0f*(float)sin(fTime*0.2));
	glRotatef(angle, 0.0f, 1.0f, 0.0f); 
	glTranslatef(0.0f,-0.6f,0.0f);
	glEnable(GL_TEXTURE_2D);
	gTexExample.Use();
	glPushMatrix();
	glTranslatef(0.0f,-0.1f,0.0f);
	glScalef(1.5,1.5,1.5);
	drawRec(RECURSION_LEVELS);
	glPopMatrix();
	gTexExample2.Use(); 
	glBegin(GL_QUADS);
	glColor3f(1,1,1);
	glTexCoord2f( 0, 0 );
	glVertex3f(-2.25f,0.0f,-2.25f);
	glTexCoord2f( 1, 0 );
	glVertex3f(2.25f,0.0f,-2.25f);
	glTexCoord2f( 1, 1);
	glVertex3f(2.25f,0.0f,2.25f);
	glTexCoord2f( 0, 1 );
	glVertex3f(-2.25f,0.0f,2.25f);
	glEnd();
	glPopMatrix();
}
